@@ -127,6 +127,8 @@ static void kernel_combo_window_hbox_visibility(GtkComboBox *kernel_combo, GtkWi
 }
 static void kernel_combo_sigma_visibility(GtkComboBox *kernel_combo, GtkWidget *wid) {
 	switch (gtk_combo_box_get_active(GTK_COMBO_BOX(kernel_combo))) {
+	case KERNEL_INVLINEAR:
+	case KERNEL_INVSQUARE:
 	case KERNEL_CONSTANT:
 	case KERNEL_BRUSH:
 	case KERNEL_BRUSH_FLIPPED:
@@ -190,7 +192,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	GtkWidget *dlg = gimp_dialog_new(
 		_("Interpolation"), PLUGIN_NAME,
 		NULL, 0,
-		gimp_standard_help_func,
+		0,
 		"plug-in-interpolate", GTK_STOCK_CANCEL,
 		GTK_RESPONSE_CANCEL, GTK_STOCK_OK,
 		GTK_RESPONSE_OK, NULL
@@ -206,15 +208,28 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	/*
 	 * preview
 	 */
+	GtkWidget *preview_vbox = gtk_vbox_new(FALSE, 0);
+	gtk_widget_show(preview_vbox);
+	gtk_box_pack_start(GTK_BOX(main_hbox), preview_vbox, TRUE, TRUE, 0);
+
 	InterpolateInvocation inv;
 	inv.drawable = drawable;
 	inv.vals = &new_vals;
 	GtkWidget *preview = gimp_drawable_preview_new(drawable, &new_ui_vals.preview);
-	gtk_box_pack_start(GTK_BOX(main_hbox), preview, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(preview_vbox), preview, TRUE, TRUE, 0);
 	gtk_widget_show(preview);
 	g_signal_connect_swapped(preview, "invalidated", G_CALLBACK(invoke_render), &inv);
 	invoke_render(&inv, GIMP_PREVIEW(preview));
 
+	GtkWidget *progress_check = gtk_check_button_new_with_mnemonic(_("Show progress"));
+	gtk_widget_show(progress_check);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(progress_check), new_vals.progress);
+	gtk_box_pack_start(GTK_BOX(preview_vbox), progress_check, FALSE, FALSE, 0);
+	gtk_widget_set_tooltip_text(progress_check, _(
+		"Show moving progress bars when running filter."
+	));
+	g_signal_connect(progress_check, "toggled", G_CALLBACK(gimp_toggle_button_update), &new_vals.progress);
+
 	GtkWidget *main_vbox = gtk_vbox_new(FALSE, 0);
 	gtk_widget_show(main_vbox);
 	gtk_box_pack_start(GTK_BOX(main_hbox), main_vbox, FALSE, FALSE, 0);
@@ -407,7 +422,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(sigma_spin), TRUE);
 	gtk_widget_set_tooltip_text(sigma_spin, _(
 		"If kernel is Linear or Square, S is the radius after which the weights are zero. "
-		"For Exponential it is a scale factor. For Gaussian it is the standard deviation."
+		"For Exponential it is a scale factor. For Gaussian it is the standard deviation. "
 		"For \"Inverse to the nth\" it is the exponent. "
 		"Generally, the smaller the S the 'sharper' the result."
 	));
@@ -665,7 +680,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	gtk_box_pack_start(GTK_BOX(slopesmask_hbox), slopesmask_check, FALSE, FALSE, 0);
 	gtk_widget_set_tooltip_text(slopesmask_check, _(
 		"Use a separate input mask for slopes calculation. "
-		"By default,  the main input mask is used."
+		"By default, the main input mask is used."
 	));
 	g_signal_connect_swapped(slopesmask_check, "toggled", G_CALLBACK(gimp_preview_invalidate), preview);
 	g_signal_connect(slopesmask_check, "toggled", G_CALLBACK(gimp_toggle_button_update), &new_vals.slopesmask);
@@ -696,7 +711,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	gtk_box_pack_start(GTK_BOX(slopesmask_hbox), slopesmaskwhite_check, FALSE, FALSE, 0);
 	gtk_widget_set_tooltip_text(slopesmaskwhite_check, _(
 		"Use a separate input mask for slopes calculation. "
-		"By default,  the main input mask is used."
+		"By default, the main input mask is used."
 	));
 	g_signal_connect_swapped(slopesmaskwhite_check, "toggled", G_CALLBACK(gimp_preview_invalidate), preview);
 	g_signal_connect(slopesmaskwhite_check, "toggled", G_CALLBACK(gimp_toggle_button_update), &new_vals.slopesmaskwhite);
@@ -705,11 +720,16 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	/*
 	 * slopessubsel
 	 */
+	GtkWidget *slopessubsel_hbox = gtk_hbox_new(FALSE, 10);
+	check_and_make_visible(GTK_TOGGLE_BUTTON(slopesmask_check), slopessubsel_hbox);
+	g_signal_connect(slopesmask_check, "toggled", G_CALLBACK(check_and_make_visible), slopessubsel_hbox);
+	gtk_box_pack_start(GTK_BOX(slopes_vbox), slopessubsel_hbox, FALSE, FALSE, 0);
+
 	GtkWidget *slopessubsel_check = gtk_check_button_new_with_mnemonic(_("Subtract selection"));
 	check_and_make_visible(GTK_TOGGLE_BUTTON(slopesmask_check), slopessubsel_check);
 	g_signal_connect(slopesmask_check, "toggled", G_CALLBACK(check_and_make_visible), slopessubsel_check);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(slopessubsel_check), new_vals.slopessubsel);
-	gtk_box_pack_start(GTK_BOX(slopes_vbox), slopessubsel_check, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(slopessubsel_hbox), slopessubsel_check, FALSE, FALSE, 0);
 	gtk_widget_set_tooltip_text(slopessubsel_check, _(
 		"Subtract selection from the above slopes input mask."
 	));
@@ -717,6 +737,28 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	g_signal_connect(slopessubsel_check, "toggled", G_CALLBACK(gimp_toggle_button_update), &new_vals.slopessubsel);
 
 
+	/*
+	 * linhalfsel
+	 */
+	GtkWidget *linhalfsel_label = gtk_label_new_with_mnemonic(_("Read threshold: "));
+	check_and_make_visible(GTK_TOGGLE_BUTTON(slopesmask_check), linhalfsel_label);
+	g_signal_connect(slopesmask_check, "toggled", G_CALLBACK(check_and_make_visible), linhalfsel_label);
+	gtk_box_pack_start(GTK_BOX(slopessubsel_hbox), linhalfsel_label, FALSE, FALSE, 0);
+	gtk_label_set_justify(GTK_LABEL(linhalfsel_label), GTK_JUSTIFY_RIGHT);
+
+	GtkObject *linhalfsel_adj = gtk_adjustment_new(new_vals.linhalfsel, 1, 255, 1, 0, 0);
+	GtkWidget *linhalfsel_spin = gtk_spin_button_new(GTK_ADJUSTMENT(linhalfsel_adj), 1, 0);
+	check_and_make_visible(GTK_TOGGLE_BUTTON(slopesmask_check), linhalfsel_spin);
+	g_signal_connect(slopesmask_check, "toggled", G_CALLBACK(check_and_make_visible), linhalfsel_spin);
+	gtk_box_pack_start(GTK_BOX(slopessubsel_hbox), linhalfsel_spin, FALSE, FALSE, 0);
+	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(linhalfsel_spin), TRUE);
+	gtk_widget_set_tooltip_text(linhalfsel_spin, _(
+		"Only pixels with at least this much input mask intensity will be read."
+	));
+	g_signal_connect_swapped(linhalfsel_adj, "value_changed", G_CALLBACK(gimp_preview_invalidate), preview);
+	g_signal_connect(linhalfsel_adj, "value_changed", G_CALLBACK(gimp_int_adjustment_update), &new_vals.linhalfsel);
+
+
 	/*
 	 * linkernel
 	 */
@@ -843,7 +885,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	gtk_box_pack_start(GTK_BOX(linkernel_capbase_hbox), linkernel_cap_spin, FALSE, FALSE, 0);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(linkernel_cap_spin), TRUE);
 	gtk_widget_set_tooltip_text(linkernel_cap_spin, _(
-		"The distance before which the linkernel should be \"capped\" (flat from center to this distance)."
+		"The distance before which the kernel should be \"capped\" (flat from center to this distance)."
 	));
 	g_signal_connect_swapped(linkernel_cap_adj, "value_changed", G_CALLBACK(gimp_preview_invalidate), preview);
 	g_signal_connect(linkernel_cap_adj, "value_changed", G_CALLBACK(gimp_float_adjustment_update), &new_vals.linkernel_cap);
@@ -861,7 +903,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 	gtk_box_pack_start(GTK_BOX(linkernel_capbase_hbox), linkernel_base_spin, FALSE, FALSE, 0);
 	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(linkernel_base_spin), TRUE);
 	gtk_widget_set_tooltip_text(linkernel_base_spin, _(
-		"The distance beyond which the linkernel should be zero."
+		"The distance beyond which the kernel should be zero."
 	));
 	g_signal_connect_swapped(linkernel_base_adj, "value_changed", G_CALLBACK(gimp_preview_invalidate), preview);
 	g_signal_connect(linkernel_base_adj, "value_changed", G_CALLBACK(gimp_float_adjustment_update), &new_vals.linkernel_base);
@@ -1066,6 +1108,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 		new_vals.halfsel = gtk_adjustment_get_value(GTK_ADJUSTMENT(halfsel_adj));
 
 		new_vals.window = gtk_adjustment_get_value(GTK_ADJUSTMENT(window_adj));
+		new_vals.window_y = gtk_adjustment_get_value(GTK_ADJUSTMENT(window_y_adj));
 		new_vals.sigma = gtk_adjustment_get_value(GTK_ADJUSTMENT(sigma_adj));
 		new_vals.kernel = gtk_combo_box_get_active(GTK_COMBO_BOX(kernel_combo));
 		new_vals.newlayer = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(newlayer_check));
@@ -1075,6 +1118,7 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 		new_vals.slopes = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(slopes_check));
 		new_vals.method = gtk_combo_box_get_active(GTK_COMBO_BOX(method_combo));
 		new_vals.linwindow = gtk_adjustment_get_value(GTK_ADJUSTMENT(linwindow_adj));
+		new_vals.linwindow_y = gtk_adjustment_get_value(GTK_ADJUSTMENT(linwindow_y_adj));
 		new_vals.linsigma = gtk_adjustment_get_value(GTK_ADJUSTMENT(linsigma_adj));
 		new_vals.linkernel = gtk_combo_box_get_active(GTK_COMBO_BOX(linkernel_combo));
 		new_vals.linnewlayer = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linnewlayer_check));
@@ -1088,13 +1132,27 @@ gboolean dialog(gint32 image_id, GimpDrawable * drawable,
 		new_vals.gamma = gtk_adjustment_get_value(GTK_ADJUSTMENT(gamma_adj));
 		new_vals.grid = gtk_adjustment_get_value(GTK_ADJUSTMENT(grid_adj));
 		new_vals.grid_y = gtk_adjustment_get_value(GTK_ADJUSTMENT(grid_y_adj));
+		new_vals.offs = gtk_adjustment_get_value(GTK_ADJUSTMENT(offs_adj));
+		new_vals.offs_y = gtk_adjustment_get_value(GTK_ADJUSTMENT(offs_y_adj));
 		new_vals.norm = gtk_adjustment_get_value(GTK_ADJUSTMENT(norm_adj));
 		new_vals.kernel_cap = gtk_adjustment_get_value(GTK_ADJUSTMENT(kernel_cap_adj));
 		new_vals.kernel_base = gtk_adjustment_get_value(GTK_ADJUSTMENT(kernel_base_adj));
+		new_vals.linkernel_cap = gtk_adjustment_get_value(GTK_ADJUSTMENT(linkernel_cap_adj));
+		new_vals.linkernel_base = gtk_adjustment_get_value(GTK_ADJUSTMENT(linkernel_base_adj));
 		new_vals.linnorm = gtk_adjustment_get_value(GTK_ADJUSTMENT(linnorm_adj));
 
+		new_vals.progress = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(progress_check));
+
 		new_vals.slopesmult = gtk_adjustment_get_value(GTK_ADJUSTMENT(slopesmult_adj));
+
+		new_vals.slopesmask = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(slopesmask_check));
 		gimp_int_combo_box_get_active(GIMP_INT_COMBO_BOX(slopeschanid_combo), &new_vals.slopeschanid);
+		new_vals.slopesmaskwhite = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(slopesmaskwhite_check));
+		new_vals.slopessubsel = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(slopessubsel_check));
+		new_vals.linhalfsel = gtk_adjustment_get_value(GTK_ADJUSTMENT(linhalfsel_adj));
+		new_vals.slopesweights = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(slopesweights_check));
+
+		new_vals.progress = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(progress_check));
 
 		memcpy(vals, &new_vals, sizeof(new_vals));
 		memcpy(ui_vals, &new_ui_vals, sizeof(new_ui_vals));
