@@ -40,7 +40,7 @@ static GimpParamDef script_args[] = {
 	{ GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
 	{ GIMP_PDB_IMAGE, "image", "Input image" },
 	{ GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
-	{ GIMP_PDB_CHANNEL, "chanid", "Input mask" },
+	{ GIMP_PDB_INT32, "chanid", "Input mask" },
 	{ GIMP_PDB_INT8, "usesel", "Subtract selection from input mask" },
 	{ GIMP_PDB_INT32, "halfsel", "Read only pixels with input mask >= halfsel" },
 
@@ -74,7 +74,6 @@ static GimpParamDef script_args[] = {
 	{ GIMP_PDB_FLOAT, "linnorm", "Norm for distance computation  (<0 means maximum distance)." },
 
 	{ GIMP_PDB_FLOAT, "slopesmult", "Artificial 'steepening' or 'flattening' of the slopes surface." },
-
 	{ GIMP_PDB_INT32, "window_y", "Vertical size of window, around each pixel, for taking weighted average" },
 	{ GIMP_PDB_INT32, "linwindow_y", "Vertical size of window for slopes estimation" },
 
@@ -82,12 +81,14 @@ static GimpParamDef script_args[] = {
 	{ GIMP_PDB_FLOAT, "kernel_base", "Kernel base distance." },
 	{ GIMP_PDB_FLOAT, "linkernel_cap", "Slopes kernel cap distance." },
 	{ GIMP_PDB_FLOAT, "linkernel_base", "Slopes kernel base distance." },
+
 	{ GIMP_PDB_INT8, "slopesmask", "Use separate slopes mask?" },
 	{ GIMP_PDB_INT32, "slopeschanid", "Slopes mask ID" },
 	{ GIMP_PDB_INT8, "slopesmaskwhite", "Use separate, white slopes mask?" },
 	{ GIMP_PDB_INT8, "slopessubsel", "Subtract selection from slopes mask?" },
 	{ GIMP_PDB_INT32, "linhalfsel", "For calculating slopes, read only pixels with slopes-mask value >= halfsel" },
 	{ GIMP_PDB_INT8, "slopesweights", "Assign a weight to each slopes sample" },
+	{ GIMP_PDB_INT8, "progress", "Show progress bars" },
 };
 
 const PlugInVals default_vals = {
@@ -139,10 +140,12 @@ const PlugInVals default_vals = {
 	TRUE,  /* slopessubsel */
 	255,  /* linhalfsel */
 	FALSE,  /* slopesweights */
+	TRUE,  /* progress */
 };
 
 const PlugInUIVals default_ui_vals = {
 	TRUE,  /* preview */
+	FALSE,  /* sepaxes */
 };
 
 static PlugInVals vals;
@@ -159,19 +162,8 @@ GimpPlugInInfo PLUG_IN_INFO = {
 MAIN ()
 
 static void query(void) {
-	gchar *help_path;
-	gchar *help_uri;
-
 	gimp_plugin_domain_register(PLUGIN_NAME, LOCALEDIR);
 
-	help_path = g_build_filename(DATADIR, "help", NULL);
-	help_uri = g_filename_to_uri(help_path, NULL, NULL);
-	g_free(help_path);
-
-	/* help is in tooltips */
-	/*gimp_plugin_help_register
-	    ("http://developer.gimp.org/plug-in-template/help", help_uri);*/
-
 	gimp_install_procedure(PROCEDURE_NAME,
 			       "Interpolation",
 			       "Does interpolation using planes and weighted averages",
@@ -221,51 +213,54 @@ static void run(const gchar *name,
 	} else switch (run_mode) {
 
 	case GIMP_RUN_NONINTERACTIVE:
-		if (n_params>=4) vals.chanid = param[3].data.d_channel;
-		if (n_params>=5) vals.usesel = param[4].data.d_int8;
-		if (n_params>=6) vals.halfsel = param[5].data.d_int32;
-		if (n_params>=7) vals.window = param[6].data.d_int32;
-		if (n_params>=8) vals.sigma = param[7].data.d_float;
-		if (n_params>=9) vals.kernel = param[8].data.d_int32;
+		if (n_params>=4) vals.chanid =    param[3].data.d_int32;
+		if (n_params>=5) vals.usesel =    param[4].data.d_int8;
+		if (n_params>=6) vals.halfsel =   param[5].data.d_int32;
+		if (n_params>=7) vals.window =    param[6].data.d_int32;
+		if (n_params>=8) vals.sigma =     param[7].data.d_float;
+		if (n_params>=9) vals.kernel =    param[8].data.d_int32;
 		if (n_params>=10) vals.newlayer = param[9].data.d_int8;
 
-		if (n_params>=11) vals.slopes = param[10].data.d_int8;
-		if (n_params>=12) vals.method = param[11].data.d_int32;
-		if (n_params>=13) vals.linwindow = param[12].data.d_int32;
-		if (n_params>=14) vals.linsigma = param[13].data.d_float;
-		if (n_params>=15) vals.linkernel = param[14].data.d_int32;
+		if (n_params>=11) vals.slopes =      param[10].data.d_int8;
+		if (n_params>=12) vals.method =      param[11].data.d_int32;
+		if (n_params>=13) vals.linwindow =   param[12].data.d_int32;
+		if (n_params>=14) vals.linsigma =    param[13].data.d_float;
+		if (n_params>=15) vals.linkernel =   param[14].data.d_int32;
 		if (n_params>=16) vals.linnewlayer = param[15].data.d_int8;
 
 		if (n_params>=17) vals.changesel = param[16].data.d_int8;
-		if (n_params>=18) vals.lowthres = param[17].data.d_float;
+		if (n_params>=18) vals.lowthres =  param[17].data.d_float;
 		if (n_params>=19) vals.highthres = param[18].data.d_float;
-		if (n_params>=20) vals.logthres = param[19].data.d_int8;
-		if (n_params>=21) vals.multmask = param[20].data.d_int8;
+		if (n_params>=20) vals.logthres =  param[19].data.d_int8;
+		if (n_params>=21) vals.multmask =  param[20].data.d_int8;
 
 		if (n_params>=22) vals.all_white_input = param[21].data.d_int8;
 
 		if (n_params>=23) vals.gamma_compensation = param[22].data.d_int8;
-		if (n_params>=24) vals.gamma = param[23].data.d_float;
-		if (n_params>=25) vals.grid = param[24].data.d_int32;
-		if (n_params>=26) vals.grid_y = param[25].data.d_int32;
-		if (n_params>=27) vals.norm = param[26].data.d_float;
-		if (n_params>=28) vals.linnorm = param[27].data.d_float;
+		if (n_params>=24) vals.gamma =   param[23].data.d_float;
+		if (n_params>=25) vals.grid =    param[24].data.d_int32;
+		if (n_params>=26) vals.grid_y =  param[25].data.d_int32;
+		if (n_params>=27) vals.offs =    param[26].data.d_int32;
+		if (n_params>=28) vals.offs_y =  param[27].data.d_int32;
+		if (n_params>=29) vals.norm =    param[28].data.d_float;
+		if (n_params>=30) vals.linnorm = param[29].data.d_float;
 
-		if (n_params>=29) vals.slopesmult = param[28].data.d_float;
-		if (n_params>=30) vals.window_y = param[29].data.d_int32;
-		if (n_params>=31) vals.linwindow_y = param[30].data.d_int32;
+		if (n_params>=31) vals.slopesmult =  param[30].data.d_float;
+		if (n_params>=32) vals.window_y =    param[31].data.d_int32;
+		if (n_params>=33) vals.linwindow_y = param[32].data.d_int32;
 
-		if (n_params>=32) vals.kernel_cap = param[31].data.d_float;
-		if (n_params>=33) vals.kernel_base = param[32].data.d_float;
-		if (n_params>=34) vals.linkernel_cap = param[33].data.d_float;
-		if (n_params>=35) vals.linkernel_base = param[34].data.d_float;
+		if (n_params>=34) vals.kernel_cap =     param[33].data.d_float;
+		if (n_params>=35) vals.kernel_base =    param[34].data.d_float;
+		if (n_params>=36) vals.linkernel_cap =  param[35].data.d_float;
+		if (n_params>=37) vals.linkernel_base = param[36].data.d_float;
 
-		if (n_params>=36) vals.slopesmask = param[35].data.d_int8;
-		if (n_params>=37) vals.slopeschanid = param[36].data.d_int32;
-		if (n_params>=38) vals.slopesmaskwhite = param[37].data.d_int8;
-		if (n_params>=39) vals.slopessubsel = param[38].data.d_int8;
-		if (n_params>=40) vals.linhalfsel = param[39].data.d_int32;
-		if (n_params>=41) vals.slopesweights = param[40].data.d_int8;
+		if (n_params>=38) vals.slopesmask =      param[37].data.d_int8;
+		if (n_params>=39) vals.slopeschanid =    param[38].data.d_int32;
+		if (n_params>=40) vals.slopesmaskwhite = param[39].data.d_int8;
+		if (n_params>=41) vals.slopessubsel =    param[40].data.d_int8;
+		if (n_params>=42) vals.linhalfsel =      param[41].data.d_int32;
+		if (n_params>=43) vals.slopesweights =   param[42].data.d_int8;
+		if (n_params>=44) vals.progress =        param[43].data.d_int8;
 
 		/*if (vals.random_seed) vals.seed = g_random_int ();*/
 		break;
