@@ -114,7 +114,7 @@ static void slopes_grid(GimpDrawable *drawable, GimpDrawable *mask, GimpDrawable
                         gint halfsel, gint linhalfsel,
                         InterpolateMethod method,
                         gfloat *g,
-                        GimpPreview *preview,
+                        GimpPreview *preview, gboolean progress,
                         gfloat *z, gfloat *w);
 
 inline static void unknown_pixel(GimpImageType type, guchar *pixel);
@@ -1039,7 +1039,7 @@ static void slopes_grid(GimpDrawable *drawable, GimpDrawable *mask, GimpDrawable
                         gfloat slopesmult,
                         gint halfsel, gint linhalfsel,
                         InterpolateMethod method,
-                        gfloat *g, GimpPreview *preview,
+                        gfloat *g, GimpPreview *preview, gboolean progress,
                         gfloat *z, gfloat *ws) {
 	gint gwidth = gright-gleft;
 	gint gheight = gbottom-gtop;
@@ -1053,7 +1053,7 @@ static void slopes_grid(GimpDrawable *drawable, GimpDrawable *mask, GimpDrawable
 	Ring mring; ring_init(&mring, mask, left, top, right, bottom, kernel->width/2, kernel->height/2);
 	Ring smring; ring_init(&smring, slopesmask, left, top, right, bottom, kernel->width/2, kernel->height/2);
 
-	if (!preview) gimp_progress_init(_("Calculating slopes..."));
+	if (!preview && progress) gimp_progress_init(_("Calculating slopes..."));
 	gint y;
 	for (y = 0 ; y < gheight ; ++y) {
 		guchar *row = ring_row(&ring);
@@ -1136,7 +1136,7 @@ static void slopes_grid(GimpDrawable *drawable, GimpDrawable *mask, GimpDrawable
 			}
 		}
 
-		if (!preview && !(y%4)) {
+		if (!preview && progress && !(y%4)) {
 			gimp_progress_update(1.0*(y+1)/gheight);
 		}
 
@@ -1148,7 +1148,7 @@ static void slopes_grid(GimpDrawable *drawable, GimpDrawable *mask, GimpDrawable
 			ring_rotate(&ring); ring_rotate(&mring); ring_rotate(&smring);
 		}
 	}
-	if (!preview) gimp_progress_end();
+	if (!preview && progress) gimp_progress_end();
 	ring_fini(&ring);
 }
 
@@ -1220,6 +1220,8 @@ inline static gfloat wsum2sel(gfloat w, gfloat lt, gfloat ht, gboolean logthres)
 }
 
 static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id, GimpPreview *preview) {
+	if (!preview && !opts->progress) gimp_progress_init(_("Interpolating..."));
+
 	GimpImageType drawable_type = gimp_drawable_type(drawable->drawable_id);
 	gint channels = gimp_drawable_bpp(drawable->drawable_id);
 
@@ -1264,7 +1266,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 	 */
 	gint tmp_mask_id = -1;
 	GimpDrawable *mask = 0;
-	if (!preview) gimp_progress_init(_("Creating mask..."));
+	if (!preview && opts->progress) gimp_progress_init(_("Creating mask..."));
 
 	if (!opts->all_white_input && gimp_drawable_is_valid(opts->chanid) && gimp_drawable_get_image(opts->chanid)==image_id) {
 		if (opts->usesel) {
@@ -1284,7 +1286,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 		subtract_selection(mask, left, top, width, height);
 	}
 
-	if (!preview) gimp_progress_end();
+	if (!preview && opts->progress) gimp_progress_end();
 
 	gint mchannels = gimp_drawable_bpp(mask->drawable_id);
 
@@ -1356,7 +1358,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 	 * make kernel
 	 */
 	InterpolateKernel kernel;
-	kernel_init(&kernel, opts->kernel, opts->window, opts->window_y, opts->sigma, opts->norm, opts->kernel_cap, opts->kernel_base, !preview);
+	kernel_init(&kernel, opts->kernel, opts->window, opts->window_y, opts->sigma, opts->norm, opts->kernel_cap, opts->kernel_base, !preview && opts->progress);
 	gint fpc = 0;  /* floats per channel */
 	switch (opts->method) {
 	case METHOD_OLD_CENTER: fpc = 3; break;
@@ -1365,7 +1367,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 	case METHOD_LEASTSQ: fpc = 3; break;
 	case METHOD_QUADRATIC: fpc = 6; break;
 	}
-	gfloat *nl = nonlinear_new(kernel.width, kernel.height, fpc, !preview);
+	gfloat *nl = nonlinear_new(kernel.width, kernel.height, fpc, !preview && opts->progress);
 
 
 	/*
@@ -1389,11 +1391,11 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 	gfloat *z=0, *ws=0;
 	if (opts->slopes && gwidth>0 && gheight>0) {
 		InterpolateKernel linkernel;
-		kernel_init(&linkernel, opts->linkernel, opts->linwindow, opts->linwindow_y, opts->linsigma, opts->linnorm, opts->linkernel_cap, opts->linkernel_base, !preview);
+		kernel_init(&linkernel, opts->linkernel, opts->linwindow, opts->linwindow_y, opts->linsigma, opts->linnorm, opts->linkernel_cap, opts->linkernel_base, !preview && opts->progress);
 
-		gfloat *linnl = nonlinear_new(linkernel.width, linkernel.height, fpc, !preview);
+		gfloat *linnl = nonlinear_new(linkernel.width, linkernel.height, fpc, !preview && opts->progress);
 
-		if (!preview) gimp_progress_init(_("Creating mask..."));
+		if (!preview && opts->progress) gimp_progress_init(_("Creating mask..."));
 
 		gint tmp_slopesmask_id = -1;
 		GimpDrawable *slopesmask = mask;
@@ -1414,7 +1416,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 			}
 		}
 
-		if (!preview) gimp_progress_end();
+		if (!preview && opts->progress) gimp_progress_end();
 
 
 		z = g_new(gfloat, fpc*channels*gwidth*gheight);
@@ -1429,7 +1431,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 		            opts->slopesmult,
 		            opts->halfsel, opts->linhalfsel,
 		            opts->method,
-		            g, preview,
+		            g, preview, opts->progress,
 		            z, ws);
 
 		if (slopesmask!=mask) {
@@ -1496,7 +1498,7 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 	/*
 	 * do the interpolation
 	 */
-	if (!preview) gimp_progress_init(_("Interpolating..."));
+	if (!preview && opts->progress) gimp_progress_init(_("Interpolating..."));
 	for (; ring_loop(&in) && ring_loop(&inm) ; ring_rotate(&in), ring_rotate(&inm)) {
 		guchar *inmrow = 0;
 		if (changesel) {
@@ -1838,11 +1840,11 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 			gimp_pixel_rgn_set_row(&outs,outsrow,left,in.y,right-left);
 		}
 
-		if (!preview && !(in.y%4)) {
+		if (!preview && opts->progress && !(in.y%4)) {
 			gimp_progress_update(1.0*(in.y-top+1)/(bottom-top));
 		}
 	}
-	if (!preview) gimp_progress_end();
+	if (!preview && opts->progress) gimp_progress_end();
 
 
 	/*
@@ -1933,6 +1935,8 @@ static void interpolate(GimpDrawable *drawable, PlugInVals *opts, gint image_id,
 	if (g) {
 		gamma_free(g);
 	}
+
+	if (!preview && !opts->progress) gimp_progress_end();
 }
 
 void render(GimpDrawable *drawable, PlugInVals *vals, GimpPreview *preview) {
