@@ -18,30 +18,32 @@ typedef struct Ring {
 	gint rowcount;
 	guchar *rows;
 	gint y;
-	gint window;
+	gint windowx;
+	gint windowy;
 } Ring;
 
-inline static void ring_init(Ring *ring, GimpDrawable *dr, gint left, gint top, gint right, gint bottom, gint window) {
+inline static void ring_init(Ring *ring, GimpDrawable *dr, gint left, gint top, gint right, gint bottom, gint windowx, gint windowy) {
 	ring->left = left;
 	ring->top = top;
 	ring->right = right;
 	ring->bottom = bottom;
 
-	ring->inleft = MAX(left-window,0);
-	ring->intop = MAX(top-window,0);
-	ring->inwidth = MIN(right+window,dr->width)-ring->inleft;
-	ring->inheight = MIN(bottom+window,dr->height)-ring->intop;
+	ring->inleft = MAX(left-windowx,0);
+	ring->intop = MAX(top-windowy,0);
+	ring->inwidth = MIN(right+windowx,dr->width)-ring->inleft;
+	ring->inheight = MIN(bottom+windowy,dr->height)-ring->intop;
 
 	ring->dr = dr;
 
 	ring->type = gimp_drawable_type(dr->drawable_id);
 	ring->bpp = gimp_drawable_bpp(dr->drawable_id);
 	ring->bpr = ring->inwidth*ring->bpp;
-	ring->rowbase = top-window;
-	ring->rowcount = (1+2*window);
+	ring->rowbase = top-windowy;
+	ring->rowcount = (1+2*windowy);
 	ring->rows = g_new(guchar, ring->rowcount*ring->bpr);
 	ring->y = top;
-	ring->window = window;
+	ring->windowx = windowx;
+	ring->windowy = windowy;
 
 	gimp_pixel_rgn_init(&ring->pr, dr,
 		ring->inleft,
@@ -50,8 +52,8 @@ inline static void ring_init(Ring *ring, GimpDrawable *dr, gint left, gint top,
 		ring->inheight,
 	FALSE, FALSE);
 	gint y;
-	for (y = MAX(top-window,0) ;
-	     y <= MIN(top+window,dr->height-1) ; ++y) {
+	for (y = MAX(top-windowy,0) ;
+	     y <= MIN(top+windowy,dr->height-1) ; ++y) {
 		gimp_pixel_rgn_get_row(&ring->pr,
 			ring->rows + (y-ring->rowbase)*ring->bpr,
 			ring->inleft, y, ring->inwidth
@@ -69,14 +71,30 @@ inline static gboolean ring_loop(Ring *ring) {
 
 inline static void ring_rotate(Ring *ring) {
 	++ring->y;
-	if (ring->y+ring->window < ring->dr->height) {
+	if (ring->y+ring->windowy < ring->dr->height) {
 		gimp_pixel_rgn_get_row(&ring->pr,
-			ring->rows + ((ring->y+ring->window-ring->rowbase)%ring->rowcount)*ring->bpr,
-			ring->inleft, ring->y+ring->window, ring->inwidth
+			ring->rows + ((ring->y+ring->windowy-ring->rowbase)%ring->rowcount)*ring->bpr,
+			ring->inleft, ring->y+ring->windowy, ring->inwidth
 		);
 	}
 }
 
-inline static guchar *ring_row(Ring *ring, gint y) {
-	return ring->rows + ( ((y-ring->rowbase)%ring->rowcount)*ring->bpr + (ring->left-ring->inleft)*ring->bpp );
+inline static guchar *ring_row(Ring *ring) {
+	return ring->rows + ( ((ring->y-ring->rowbase)%ring->rowcount)*ring->bpr + (ring->left-ring->inleft)*ring->bpp );
+}
+inline static guchar *ring_row_delta(Ring *ring, gint deltay) {
+	return ring->rows + ( ((ring->y+deltay-ring->rowbase)%ring->rowcount)*ring->bpr + (ring->left-ring->inleft)*ring->bpp );
+}
+
+inline static gint ring_min_deltay(Ring *ring) {
+	return MAX(-ring->windowy, -ring->y);
+}
+inline static gint ring_max_deltay(Ring *ring) {
+	return MIN(ring->windowy, ring->dr->height-1-ring->y);
+}
+inline static gint ring_min_deltax(Ring *ring, gint x) {
+	return MAX(-ring->windowx, -x);
+}
+inline static gint ring_max_deltax(Ring *ring, gint x) {
+	return MIN(ring->windowx, ring->dr->width-1-x);
 }
